from typing import Optional, List, Union

from endstone.logger import Logger
from endstone.server import Server


class PluginDescription:
    def __init__(
        self,
        name: str,
        version: str,
        description: Optional[str] = None,
        authors: Optional[List[str]] = None,
        prefix: Optional[str] = None,
        *args,
        **kwargs
    ) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def version(self) -> str: ...
    @property
    def full_name(self) -> str: ...
    @property
    def description(self) -> Optional[str]: ...
    @property
    def authors(self) -> Optional[List[str]]: ...
    @property
    def prefix(self) -> Optional[str]: ...

class PluginLoader:
    def __init__(self, server: Server) -> None: ...
    def load_plugins(self, directory: str) -> List[Plugin]: ...
    def enable_plugin(self, plugin: Plugin) -> None: ...
    def disable_plugin(self, plugin: Plugin) -> None: ...
    @property
    def server(self) -> Server: ...

class Plugin:
    def __init__(self) -> None: ...
    def on_disable(self) -> None: ...
    def on_enable(self) -> None: ...
    def on_load(self) -> None: ...
    @property
    def description(self) -> PluginDescription: ...
    @property
    def enabled(self) -> bool: ...
    @property
    def logger(self) -> Logger: ...
    @property
    def plugin_loader(self) -> PluginLoader: ...
    @property
    def server(self) -> Server: ...

class PluginManager:
    def register_loader(self, loader: PluginLoader) -> None: ...
    def get_plugin(self, name: str) -> Plugin: ...
    @property
    def plugins(self) -> List[Plugin]: ...
    def is_plugin_enabled(self, plugin: Union[str, Plugin]) -> bool: ...
    def load_plugins(self, directory: str) -> List[Plugin]: ...
    def enable_plugin(self, plugin: Plugin) -> None: ...
    def disable_plugin(self, plugin: Plugin) -> None: ...
    def disable_plugins(self) -> None: ...
    def clear_plugins(self) -> None: ...
